@@ -3,9 +3,9 @@ use strict;
 use warnings;
 use Test::Trap ();        # load as early as possible to override CORE::exit
 
-our $VERSION = '0.50';
+our $VERSION = '0.51';
 
-use base qw(Exporter);
+use parent 'Exporter';
 
 use Carp ();
 use Exporter ();
@@ -19,7 +19,7 @@ our $Debug = $ENV{TEST_SPEC_DEBUG} || 0;
 
 our @EXPORT      = qw(runtests
                       describe xdescribe context xcontext it xit they xthey
-                      before after spec_helper
+                      before after around yield spec_helper
                       *TODO share shared_examples_for it_should_behave_like );
 our @EXPORT_OK   = ( @EXPORT, qw(DEFINITION_PHASE EXECUTION_PHASE $Debug) );
 our %EXPORT_TAGS = ( all => \@EXPORT_OK,
@@ -70,7 +70,7 @@ sub import {
 
   eval qq{
     package $callpkg;
-    use base 'Test::Spec';
+    use parent 'Test::Spec';
     # allow Test::Spec usage errors to be reported via Carp
     our \@CARP_NOT = qw($callpkg);
   };
@@ -203,7 +203,7 @@ sub describe(@) {
     $container = $_Current_Context->context_lookup;
   }
   else {
-    $container = $_Package_Contexts{$package} ||= {};
+    $container = $_Package_Contexts{$package} ||= Test::Spec::_ixhash();
   }
 
   __PACKAGE__->_accumulate_examples({
@@ -215,6 +215,22 @@ sub describe(@) {
   });
 }
 
+# around CODE
+sub around(&) {
+  my $package = caller;
+  my $code = pop;
+  if (ref($code) ne 'CODE') {
+    Carp::croak "expected subroutine reference as last argument";
+  }
+  my $context = _autovivify_context($package);
+  push @{ $context->around_blocks }, { code => $code };
+}
+
+# yield
+sub yield() {
+  $Test::Spec::Context::yield->();
+}
+
 # make context() an alias for describe()
 sub context(@);
 BEGIN { *context = \&describe }
@@ -731,6 +747,27 @@ respectively.  The default is "each".
 
 C<after "all"> blocks run I<after> C<after "each"> blocks.
 
+=item around CODE
+
+Defines code to be run around tests in the current describe block are
+run. This code must call C<yield>..
+
+  our $var = 0;
+
+  describe "Something" => sub {
+    around {
+      local $var = 1;
+      yield;
+    };
+
+    it "should have localized var" => sub {
+      is $var, 1;
+    };
+  };
+
+=item yield
+
+Runs examples in context of C<around> block.
 
 =item shared_examples_for DESCRIPTION => CODE
 
