@@ -7,11 +7,10 @@
 #
 ########################################################################
 #
-# override Test::Harness's insistence on using "perl -w"
-BEGIN { $^W = 0 }
-BEGIN { open(STDERR, ">&STDOUT") }  # 2>&1
 
 use Test::Spec;
+use FindBin qw($Bin);
+BEGIN { require "$Bin/test_helper.pl" };
 
 describe "Test::Spec" => sub {
   it "should trap die message" => sub {
